from logging import INFO
from aiohttp.client import ClientSession
import asyncclick as click
from asyncclick.core import Context
import coloredlogs

from sodisys.sodisys import Sodisys
from sodisys.cli.commands import checkin_status
from sodisys.cli.commands import user_info


@click.group()
@click.version_option()
@click.option("username", "--user", help="Username used for login.", required=True)
@click.option("password", "--password", help="Password used for login.", required=True)
@click.pass_context
async def cli(  # noqa: PLR0913
    ctx: Context,
    username: str,
    password: str,
) -> None:
    coloredlogs.install(level=INFO)

    ctx.ensure_object(dict)
    ctx.obj["username"] = username
    ctx.obj["password"] = password

    session = ClientSession()
    sodisys = Sodisys(session)

    await sodisys.login(username, password)

    ctx.obj["session"] = session
    ctx.obj["sodisys"] = sodisys


cli.add_command(checkin_status)
cli.add_command(user_info)


@cli.result_callback()
@click.pass_context
async def disconnect(  # noqa: PLR0913
    ctx: Context,
    result: None,  # noqa: ARG001
    username: str,  # noqa: ARG001
    password: str,  # noqa: ARG001
) -> None:
    session: ClientSession = ctx.obj["session"]

    await session.close()



import datetime
import asyncclick as click
from asyncclick import Context
from rich import print


@click.command()
@click.pass_context
async def checkin_status(ctx: Context) -> None:
    live = await ctx.obj["sodisys"].get_live()

    if live.last_slot is None:
        print("No checkin data available for today")
        return

    print(
        "checkin at",
        live.last_slot.get_checkin_timestamp(
            datetime.timezone(datetime.timedelta(hours=2))
        ),
    )
    print(
        "checkout at",
        live.last_slot.get_checkout_timestamp(
            datetime.timezone(datetime.timedelta(hours=2))
        ),
    )


@click.command()
@click.pass_context
async def user_info(ctx: Context) -> None:
    data = await ctx.obj["sodisys"].get_data()

    print("first name:", data.user_details.firstname)
    print("last name:", data.user_details.lastname)



from pydantic import BaseModel
import datetime


class LastSlot(BaseModel):
    in_time: datetime.time | None
    out_time: datetime.time | None
    date: datetime.date

    def get_checkin_timestamp(
        self, timezone: datetime.timezone
    ) -> datetime.datetime | None:
        if self.in_time is None:
            return None
        return datetime.datetime.combine(self.date, self.in_time).replace(
            tzinfo=timezone
        )

    def get_checkout_timestamp(
        self, timezone: datetime.timezone
    ) -> datetime.datetime | None:
        if self.out_time is None:
            return None
        return datetime.datetime.combine(self.date, self.out_time).replace(
            tzinfo=timezone
        )


class LiveResponse(BaseModel):
    last_slot: LastSlot | None



import logging
from aiohttp import ClientSession

from sodisys.rest_api import RestApi
from sodisys.auth import SodisysAuthorization
from sodisys.rest_api.model.live import LiveResponse
from sodisys.rest_api.model.data import DataResponse

_LOGGER = logging.getLogger(__name__)


class Sodisys:
    session: ClientSession
    authorization: SodisysAuthorization
    rest_api: RestApi

    def __init__(self, session: ClientSession) -> None:
        self.session = session
        self.authorization = SodisysAuthorization(session)
        self.rest_api = RestApi(session, self.authorization)

    async def login(self, username: str, password: str) -> None:
        await self.authorization.authorize(username, password)
        _LOGGER.debug("authorization successful.")

    async def get_live(self) -> LiveResponse:
        return await self.rest_api.get_live()

    async def get_data(self) -> DataResponse:
        return await self.rest_api.get_data()



from attr import dataclass
from abc import ABC
from aiohttp import ClientSession


@dataclass
class SodisysSession:
    jwt_token: str


class SodisysAuthorization(ABC):
    session: ClientSession
    _sodisys_session: SodisysSession | None = None

    def __init__(self, session: ClientSession) -> None:
        self.session = session

    async def authorize(self, username: str, password: str) -> None:
        params = {
            "username": username,
            "password": password,
        }

        response = await self.session.post(
            "https://appapi.app.sodisys.eu/public/a/login", json=params
        )
        response_json = await response.json()
        token = response_json["jwt_token"]

        self._sodisys_session = SodisysSession(token)

    def get_access_token(self) -> str:
        if not self._sodisys_session:
            raise NotAuthorizedError

        return self._sodisys_session.jwt_token


class NotAuthorizedError(Exception):
    """Not authorized.

    Did you forget to call Authorization.authorize()?
    """



from sodisys.auth import SodisysAuthorization
from aiohttp import ClientSession

from sodisys.rest_api.model.data import DataResponse
from sodisys.rest_api.model.live import LiveResponse


class RestApi:
    session: ClientSession
    authorization: SodisysAuthorization

    def __init__(self, session: ClientSession, authorization: SodisysAuthorization):
        self.session = session
        self.authorization = authorization

    async def get_live(self) -> LiveResponse:
        url = "https://sysapi-aws.app.sodisys.eu/sta_client/proplan/live"
        headers = self._headers()
        async with self.session.get(url, headers=headers) as response:
            return LiveResponse.model_validate_json(await response.text())

    async def get_data(self) -> DataResponse:
        url = "https://appapi.app.sodisys.eu/public/auth/sso/data"
        headers = self._headers()
        async with self.session.post(url, headers=headers) as response:
            return DataResponse.model_validate_json(await response.text())

    def _headers(self):
        return {
            "authorization": f"Bearer {self.authorization.get_access_token()}",
            "accept": "application/json",
            "referer": "https://app.sodisys.net",
            "zone": "Europe/London",
            "timezone": "UTC",
            "app-version": "1",
        }
